/*
 *          Copyright (c) mjh-EDV Beratung, 1996-1998
 *     mjh-EDV Beratung - 63263 Neu-Isenburg - Rosenstrasse 12
 *          Tel +49 6102 328279 - Fax +49 6102 328278
 *                Email info@mjh.teddy-net.com
 *
 *   Author: Jordan Hrycaj <jordan@mjh.teddy-net.com>
 *
 *   $Id: cipher.h,v 1.2 1998/12/14 23:43:46 root Exp $
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Library General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 */

#ifndef __CIPHER_H__
#define __CIPHER_H__

/* The list of encryption protocols available */

#define CIPHER_PLAIN        /* just a fake: for testing the cbc frame, only */
#define CIPHER_BLOWFISH     /* o.k. and fast, no export restrictions */
#undef  CIPHER_3DES         /* o.k., us export restrictions */
#undef  CIPHER_IDEA         /* secure, slower than blowfish, copyrighted */

/* The list of frames, supported to prevent against an insertion attack.
   Here, cipher 64 bit or 128 bit block chaining mode is used. */

#define CRC16_FRAME /* 16 bit crc frame, for testing, only */ 
#define   MD5_FRAME /* md5/16 bit instead of crc, is o.k. */
#define   SHA_FRAME /* sha/16 bit instead of crc, is o.k. */

typedef void (*crypto_fn) (void *, unsigned char *out, const unsigned char *in);

/* cipher and frame instancees */
typedef struct _cipher_desc {

  unsigned keylen ;		/* bytes needed for the key */
  unsigned blocklen ;		/* block size in bytes */
  crypto_fn crypt ;		/* the block ciper to apply */
  void *class ;			/* opaque referece to the class */
  char context [1] ;		/* the rest is the external descriptor */
} cipher_desc ;


typedef struct _crc_fns {
  void           (*first) (void*, const char *buf, unsigned len);
  void            (*next) (void*, const char *buf, unsigned len);
  unsigned long (*result) (void*);
} crc_fns ;

typedef struct _frame_desc {
  unsigned offset ;		/* to extract 4 bytes from hash string */
  unsigned long serial ;	/* crc is chained with the last block */
  crc_fns crc ;			/* the crc set of functins to apply */
  void *class ;			/* opaque referece to the class */
  char context [1] ;		/* the rest is the external descriptor */
} frame_desc ;


/* user interface to crypto function */
#define XCRYPT(d,out,in)      (*(d)->crypt) ((d)->context, out, in)

/* user interface to crc/md functions */
#define XCRCFIRST( d,buf,len) (*(d)->crc.first) ((d)->context, buf, len)
#define XCRCNEXT(  d,buf,len) (*(d)->crc.next)  ((d)->context, buf, len)
#define XCRCRESULT(d)         (*(d)->crc.result)((d)->context)


/* get the classes */
extern void *find_cipher_class (const char *name);
extern void *find_frame_class (const char *name);
extern unsigned cipher_keylen (const char *cipher) ;


/* get all classes my name <cipher-name>/<crc-name> */
extern unsigned find_classes_by_name (void **cipher_class, void **frame_class, 
				      const char *name);

/* get the instances */
extern cipher_desc *create_encryption (void *cipher_class, const char *key, unsigned len) ;
extern cipher_desc *create_decryption (void *cipher_class, const char *key, unsigned len) ;
extern frame_desc *create_frame  (void *frame_class, unsigned long start, unsigned offset) ;

/* destroy the instances */
extern void        destroy_cipher (cipher_desc *) ;
extern void        destroy_frame  (frame_desc *);

#endif /* __CIPHER_H__ */
