/*
 *          Copyright (c) mjh-EDV Beratung, 1996-1998
 *     mjh-EDV Beratung - 63263 Neu-Isenburg - Rosenstrasse 12
 *          Tel +49 6102 328279 - Fax +49 6102 328278
 *                Email info@mjh.teddy-net.com
 *
 *       Author: Jordan Hrycaj <jordan@mjh.teddy-net.com>
 *
 *   $Id: iostream.h,v 1.4 1998/12/20 18:07:35 root Exp $
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Library General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *   This module sets a generic layer on top of the standard io functions
 *   like send, recv, read and write.  It replaces the channel numbers by
 *   a generic descriptor and calls the approriate io functions.
 */


#ifndef __IOSTREAM_H__
#define __IOSTREAM_H__

/* prototype of a high level io function */
typedef int (*io_fn) (void *desc, 
		      const void *msg, unsigned len, 
		      unsigned flags) ;

/* prototype of a generic ioctl function */
typedef int (*ctl_fn) (void *desc, int request, void *arg) ;

/* prototype of a destuctor function */
typedef void (*destroy_fn) (void *desc, unsigned fd) ;

/* layer create/destroy management */
#define      IO_PUSH_EXCLUSIVELY 4
extern void *io_push (unsigned fd, unsigned size, io_fn, ctl_fn, 
		      void (*flush)(void*), 
		      destroy_fn, unsigned how);	/* never NULL */
extern void  io_pop (unsigned fd, unsigned how) ;


/* some generic control function passing */
/* control requests to be passed to the io_ctl () function
   defined in iostream */
#define IO_RESIZE_BUF       1 /* arg is (unsigned long*) */
#define IO_CHANGE_KEY       2 /* not yet */
extern int io_ctrl (unsigned fd, int request, void *arg, unsigned how) ;

/* start/stop (default == stop) management */
extern void  io_enable (unsigned fd, unsigned how) ;
extern void io_suspend (unsigned fd, unsigned how) ;


/* io top layer functions, emulates send/recv */
extern int io_send (unsigned fd, const char *buf, unsigned len, unsigned flags);
extern int io_recv (unsigned fd,       char *buf, unsigned len, unsigned flags);
extern void rd_flush (unsigned fd) ;
extern int io_shutdown (unsigned fd, unsigned flags);


/* replacement for network read/write functions */
#ifndef __IOSTREAM_PRIVATE__
#define send(fd, buf, n, flg)   io_send (fd, buf, n, flg)
#define recv(fd, buf, n, flg)   io_recv (fd, buf, n, flg)
#define shutdown(fd,flg)        io_shutdown (fd, flg)
#endif

#endif /* __IOSTREAM_H__ */
