/*
 *          Copyright (c) mjh-EDV Beratung, 1996-1998
 *     mjh-EDV Beratung - 63263 Neu-Isenburg - Rosenstrasse 12
 *          Tel +49 6102 328279 - Fax +49 6102 328278
 *                Email info@mjh.teddy-net.com
 *
 *   Author: Jordan Hrycaj <jordan@mjh.teddy-net.com>
 *
 *   $Id: messages.h,v 1.4 1998/12/20 18:59:02 root Exp $
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Library General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   PEKS - private exponent key stuff for Diffie Hellman and El Gamal
 */

#ifndef __MESSAGES_H__
#define __MESSAGES_H__


/* some magic */
#define _arg1of2(x,y) x
#define _arg2of2(x,y) y


/* extract value from error symbol */
#define errnum(sym)  (sym (_arg1of2) + 20000)
#define errstr(sym)   sym (_arg2of2)


/* concise list of all error symbols */
#define PEKS_ERRLIST(f) f(NO_PEKS_ERROR)					\
	f(CANT_OPEN_KEY_RFILE) f(CANT_OPEN_KEY_WFILE) f(KEYFILE_PARSER_ERR)	\
	f(TOO_MANY_PRIME_TRIES)	f(ELG_TEXT_2LONG) f(ELG_B64TEXT_ERR)		\
	f(RESP_VERS_MISMATCH) f(RESP_PUBKEY_MISSING)				\
	f(RESP_CODE_MISSING) f(RESP_CRC_MISSING) f(RESP_2MANY_FIELDS)		\
	f(RESP_CRCVAL_ERR) f(RESP_PUBKEY_ERR) f(RESP_PAYLOAD_ERR)		\
	f(ACCEPT_PUBKEY_ERR) f(DECODE_SRVKEY_ERR)				\
	f(MAKE_PUBKEY_ERR) f(DECODE_NO_CIPHER) f(DECODE_CLNTKEY_ERR)		\
        f(SRV_SETUP_SKEY) f(SRV_SEND_FAILED) f(SRV_INSTALL_CKEY)		\
	f(SRV_INSTALL_SKEY) f(CLNT_INSTALL_SKEY) f(CLNT_INSTALL_CKEY)		\
        f(CLNT_MAGIC_FAILED)  f(SRV_MAGIC_FAILED) f(SRV_WRONG_MAGIC)		\
        f(CBC_RECV_NULL_BLOCK) f(CBC_RECV_LEN_2SMALL) f(CBC_RECV_LEN_2LARGE)	\
        f(CBC_RECV_LEN_NOBNDRY) f(CBC_RECV_UNEXP_EOF) f(CBC_RECV_CRCERR)	\
        f(CBC_CACHERES_BLOCKED) f(CBC_UNKNOWN_CTL) f(CBC_ILLRES_VALUE)


/* no error symbol */
#define NO_PEKS_ERROR(f)	f(0, "No peks error - all o.k.")

/* peks-file.c error symbols */
#define CANT_OPEN_KEY_RFILE(f)	 f(1, "Can't open key file for reading")
#define CANT_OPEN_KEY_WFILE(f)	 f(2, "Can't open key file for writing")
#define KEYFILE_PARSER_ERR(f)	 f(3, "Illegal lines in key file")

/* peks-calc.c error symbols */
#define TOO_MANY_PRIME_TRIES(f)	f(11, "Can't generate prime/generator - tried too often") 

/* peks-handshake.c error symbols */
#define ELG_TEXT_2LONG(f)	f(31, "Argument text longer than prime module")
#define ELG_B64TEXT_ERR(f)	f(32, "Can't convert text to base 64 numeric (internal problem)") 
#define RESP_VERS_MISMATCH(f)	f(41, "Unacceptable version string in the respose text")
#define RESP_PUBKEY_MISSING(f)	f(42, "Missing 1st public key field in the response text")
#define RESP_CODE_MISSING(f)	f(43, "Missing 2nd code field in the response text")
#define RESP_CRC_MISSING(f)	f(44, "Missing 3rd crc field in the response text")
#define RESP_2MANY_FIELDS(f)	f(45, "More than 3 fields in the response text")
#define RESP_CRCVAL_ERR(f)	f(46, "Crc verification of the response text failed")
#define RESP_PUBKEY_ERR(f)	f(47, "No base64 coded public key wrapped in the response")
#define RESP_PAYLOAD_ERR(f)	f(48, "No base64 coded response text or key")

/* peks-client.c error symbols */
#define ACCEPT_PUBKEY_ERR(f)	f(51, "Can't decode public server key message")
#define DECODE_SRVKEY_ERR(f)	f(52, "Can't decode server key message")
#define CLNT_INSTALL_SKEY(f)	f(53, "Could not install session key from server")
#define CLNT_INSTALL_CKEY(f)	f(54, "Could not install client session key")
#define CLNT_MAGIC_FAILED(f)	f(55, "Could not send magic string to server")

/* peks-server.c error symbols */
#define MAKE_PUBKEY_ERR(f)	f(61, "Can't generate new server public key")
#define DECODE_NO_CIPHER(f)	f(62, "Client used wrong encryption type for prv key wrapper")
#define DECODE_CLNTKEY_ERR(f)	f(63, "Can't decode client key message")
#define SRV_SETUP_SKEY(f)	f(64, "Can't setup peks server key")
#define SRV_INSTALL_CKEY(f)	f(65, "Could not install session key from client")
#define SRV_INSTALL_SKEY(f)	f(66, "Could not install server session key")
#define SRV_SEND_FAILED(f)	f(67, "Server could not send session key")
#define SRV_MAGIC_FAILED(f)	f(68, "Could not read magic string from client")
#define SRV_WRONG_MAGIC(f)	f(69, "Client sent incompatible magic string")

/* cbc-frame.c error symbols */
#define CBC_RECV_NULL_BLOCK(f)	f(71, "Received EOF when block was expected")
#define CBC_RECV_LEN_2SMALL(f)	f(72, "Received block length value too small")
#define CBC_RECV_LEN_2LARGE(f)	f(73, "Received block length value exceeds limit")
#define CBC_RECV_LEN_NOBNDRY(f)	f(74, "Received block length value not on block boundary")
#define CBC_RECV_UNEXP_EOF(f)	f(75, "Got EOF in the middle of a block read")
#define CBC_RECV_CRCERR(f)	f(76, "CRC error")
#define CBC_CACHERES_BLOCKED(f)	f(77, "Need to flush cache before resizing")
#define CBC_UNKNOWN_CTL(f)	f(78, "Unknown control request")
#define CBC_ILLRES_VALUE(f)	f(79, "Buffer resizing argument out of range")


#endif /* __MESSAGES_H__ */
