/*
 *          Copyright (c) mjh-EDV Beratung, 1996-1998
 *     mjh-EDV Beratung - 63263 Neu-Isenburg - Rosenstrasse 12
 *          Tel +49 6102 328279 - Fax +49 6102 328278
 *                Email info@mjh.teddy-net.com
 *
 *   Author: Jordan Hrycaj <jordan@mjh.teddy-net.com>
 *
 *   $Id: peks-client.c,v 1.5 1998/12/20 14:50:11 root Exp $
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Library General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   PEKS - private exponent key stuff for Diffie Hellman and El Gamal
 */

#include "common-stuff.h"
#include "peks-internal.h"
#include "peks-random.h"
#include "messages.h"
#include "cbc-frame.h"
#include "baseXX.h"

#include <limits.h>
#include <sys/types.h>
#include <sys/socket.h>

#define __IOSTREAM_PRIVATE__ /* prevent from using send/recv auto-replaces */
#include "iostream.h"

/* ---------------------------------------------------------------------------- *
 *                  public key handshake functions                              *
 * ---------------------------------------------------------------------------- */

static unsigned
peks_client_setup (K, in, file)
     peks_key **K;
     const char *in;
     const char *file;
{
  if ((*K = accept_public_key_str (in)) == 0) 
    return errnum (ACCEPT_PUBKEY_ERR) ;
  
  /* FIXME: check whether we know the server key, already */

  return 0;
}


static char *
peks_client_encode_receiver_key (k, s, len, type)
     peks_key *k ;
     const char *s;
     unsigned len;
     const char *type;
{
  char *t, *sk, *sl ;

  if (len == 0)
    len = strlen (s) ;
  if (type == 0)
    type = "" ;

  /*
   * make the public peks encrypted receiver session key as
   *
   *	s [] := ("key:" <base64-key-bits> <cipher-type>)
   */
  sk = bin2base64 (s, len) ;
  sl  = ALLOCA (8 + strlen (sk) + strlen (type));
  sprintf (sl, "key: %s %s\n", sk, type) ;
  XFREE (sk) ;

  t = make_elg_response_key_str (k, sl, strlen (sl), 0) ;
  DEALLOCA (sl) ;

  return t ;
}


static char *
peks_client_unwrap_sender_key (N, T, in)
     unsigned *N;
     const char **T;
     const char *in;
{
  char *s, *sk, *sl;
  unsigned dummy ;

  if (N == 0)
    N = &dummy ;

  /* must be editbale in order to use strtok () */
  s = ALLOCASTRDUP (in) ;

  /* parse the sender session key from the server:
   *
   *	key: <base64-key-bits> <cipher-type>
   */

  if ((sk = strtok (s, PEKS_FS)) == 0 ||
      strcmp (sk, "key:")         != 0 ||
      (sk = strtok  (0, PEKS_FS)) == 0 ||
      (sl = strtok  (0, PEKS_FS)) == 0) {
    DEALLOCA (in);
    *N = errnum (DECODE_SRVKEY_ERR) ;
    return 0 ;
  }

  /* extract the binary key */
  sk = base64toBin (sk, N) ;

  /* dup the cypher type */
  if (T != 0)
    *T = XSTRDUP (sl) ;

  /* clean up and return */
  DEALLOCA (in);
  return sk;
}



/* ---------------------------------------------------------------------------- *
 *                  public key client handshake function                        *
 * ---------------------------------------------------------------------------- */

int
client_negotiate_session_key (cipher, socket, server, keyfile)
     const char  * cipher ;
     int           socket ;
     const char  * server ;
     const char * keyfile ;
{
  peks_key *key ;
  char *s, *u, *buf ;
  int n, l ;

  /* read the public key from the server host */
  buf = ALLOCA (CBC_IO_BUFLEN+1) ;
  if ((n = recv (socket, buf, CBC_IO_BUFLEN, 0)) < 0) {
    DEALLOCA (buf) ;
    return errno ;
  }
  buf [n] = 0 ;

  if ((n = peks_client_setup (&key, buf, keyfile)) != 0) {
    XFREE (buf) ;
    return n ;
  }

  /* FIXME: generate the receiver session key s of length l */
  s = ALLOCA (l = cbc_keylen (cipher)) ;
  setRandomBytes (s, l) ;
  
  /* install the receiver session key, here */
  if ((n = cbc_recv_init (socket, cipher, s, l)) != 0) {
    end_peks_key (key) ;
    XFREE (s);
    DEALLOCA (buf) ;
    return errnum (CLNT_INSTALL_SKEY) ;
  }
  io_enable (socket, 0) ;

  /*  wrap the client session key and send it */
  u = peks_client_encode_receiver_key (key, s, l, cipher) ;
  n = key->errnum ;
  end_peks_key (key) ;			/* not needed, any longer */
  DEALLOCA (s);
  if (u == 0) {
    DEALLOCA (buf) ;
    return n ;
  }
  send (socket, u, strlen (u), 0);
  XFREE (u) ;
  
  /* receive the sender session key from the server */
  if ((n = io_recv (socket, buf, CBC_IO_BUFLEN, 0)) < 0) {
    DEALLOCA (buf) ;
    return errno ;
  }
  buf [n] = '\0' ;

  s = peks_client_unwrap_sender_key (&l, &u, buf) ;
  DEALLOCA (buf) ;			/* not needed, any longer */
  if (s == 0)				/* in which case l == error code */
    return l ;
    
  /* install the sender session key s of length l and type t */
  n = cbc_send_init (socket, cipher, s, l) ;
  XFREE (s) ;
  if (n != 0)
    return errnum (CLNT_INSTALL_CKEY) ;
  io_enable (socket, 1) ;

  if (io_send (socket, PEKS_MAGIC, strlen (PEKS_MAGIC) + 1, 0) !=
      strlen (PEKS_MAGIC) + 1)
    return errnum (CLNT_MAGIC_FAILED) ;
  
  return 0;
}
