/*
 *          Copyright (c) mjh-EDV Beratung, 1996-1998
 *     mjh-EDV Beratung - 63263 Neu-Isenburg - Rosenstrasse 12
 *          Tel +49 6102 328279 - Fax +49 6102 328278
 *                Email info@mjh.teddy-net.com
 *
 *   Author: Jordan Hrycaj <jordan@mjh.teddy-net.com>
 *
 *   $Id: peks-random.c,v 1.1 1998/12/13 21:46:50 root Exp $
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Library General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * PEKS - private exponent key stuff for Diffie Hellman and El Gamal
 */


#include "common-stuff.h"
#include "peks-internal.h"
#include "baseXX.h"
#include <stdlib.h>

#if CHAR_BIT != 8
#error Not supporting this architecture: CHAR_BIT != 8
#endif

/* !!!! THERE IS NO REAL RANDOMNESS EXPECTED, YET !!!!! */

void
initRandomGenerator (seed)
     unsigned seed ;
{
  srand (seed) ;
}

/* !!!! THERE IS NO REAL RANDOMNESS EXPECTED, YET !!!!! */

char *
setRandomBytes (buf, len)
     char *buf ;
     unsigned len;
{
  unsigned char *p = buf ;

  /* Qoted from the Linux man page:
     
     In Numerical Recipes in C: The Art of Scientific Computing
     (William  H.  Press, Brian P. Flannery, Saul A. Teukolsky,
     William T.  Vetterling;  New  York:  Cambridge  University
     Press,  1992 (2nd ed, p. 277)), the following comments are
     made:
              "If you want to generate a random integer between 1
              and 10, you should always do it by using high-order
              bits, as in

		     j=1+(int) (10.0*rand()/(RAND_MAX+1.0));

              and never by anything resembling

                     j=1+(rand() % 10);

              (which uses lower-order bits)."
  */

  while (len --) {       
    /* calculate an integer smaller than 256.0 - 1/(RAND_MAX-1.0) */
    * p ++ = (unsigned char)(256.0 * rand () / (RAND_MAX+1.0)) ;
  }

  return buf ;
}

/* !!!! THERE IS NO REAL RANDOMNESS EXPECTED, YET !!!!! */

void
getMpzRandomNumber (ROP, bits)
     mpz_t *ROP ;
     unsigned bits;
{
  unsigned  n = (bits + 7) / 8 ;
  char *t, *s = ALLOCA (n) ;

  setRandomBytes (s, n);
  t = bin2base64 (s, n) ;
  DEALLOCA (s) ;
  base64toMpz (ROP, t) ;
  XFREE (t) ;
}

