/*
 *          Copyright (c) mjh-EDV Beratung, 1996-1998
 *     mjh-EDV Beratung - 63263 Neu-Isenburg - Rosenstrasse 12
 *          Tel +49 6102 328279 - Fax +49 6102 328278
 *                Email info@mjh.teddy-net.com
 *
 *    Author: Jordan Hrycaj <jordan@mjh.teddy-net.com>
 *
 *   $Id: plain.c,v 1.3 1998/12/20 16:38:52 root Exp $
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Library General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 */


#include "cipher.h"

/* ---------------------------------------------------------------------------- *
 *                   dummy ciphers for testing                                  *
 * ---------------------------------------------------------------------------- */

#ifdef CIPHER_PLAIN
#include <string.h>
#include "common-stuff.h"

static void
plain8 (d, out, in)
     void   *d;
     byte *out;
     byte  *in;
{
  memcpy (out, in, 8);
}

static void
plain16 (d, out, in)
     void   *d;
     byte *out;
     byte  *in;
{
  memcpy (out, in, 16);
}

static void
invert8 (d, out, in)
     void   *d;
     byte *out;
     byte  *in;
{
  memcpy (out, in, 8);
  *(u32*) out    ^= (u32)0 ;
  *(u32*)(out+4) ^= (u32)0 ;
}


static void
invert16 (d, out, in)
     void   *d;
     byte *out;
     byte  *in;
{
  memcpy (out, in, 16);
  *(u32*) out       ^= (u32)0 ;
  *(u32*)(out +  4) ^= (u32)0 ;
  *(u32*)(out +  8) ^= (u32)0 ;
  *(u32*)(out + 12) ^= (u32)0 ;
}


static int
set_key (d, buf, len)
     void       *d;
     byte     *buf;
     unsigned  len;
{
  ((char*) d) [0] = 3 ;
  ((char*) d) [1] = 2 ;
  ((char*) d) [2] = 1 ;
  return 0;
}
#endif /* CIPHER_PLAIN */


/* ---------------------------------------------------------------------------- *
 *                   dummy crc for testing                                      *
 * ---------------------------------------------------------------------------- */

#ifdef CRC16_FRAME
#include "common-stuff.h"

/* The generator polynomial for crc16, used only to verify the keys 
   stored on disk in the key file.  This algorithm is used solely
   to detect a corrupt line entry */ 
#define CRC16_POLYNOMIAL 0xA001 
#define CRC16_PRESET     0xffff

static void
crc16_init (void *Accu)
{
  *(unsigned*)Accu =  CRC16_PRESET ;
}

static void
crc16_final (void *Accu)
{
}

static byte *
crc16_read (void *Accu)
{
  unsigned accu = *(unsigned *)Accu ;
  unsigned    n = sizeof (unsigned) ;
  while (n --) {
    /* store in big endian */
    ((byte *)Accu)[n] = accu & 0xff ;
    accu >>= 8 ;
  }
  return Accu;
}

static void
crc16_write (Accu, buf, len)
     void *Accu;
     byte  *buf;
     size_t len;
{
  unsigned accu = *(unsigned *)Accu ;
  unsigned i = -1, j; 
  
  /* note that this algorithm shifts right although I find
     it more natural to shift left - it is used that way
     in some INDACTIC implementation, for example */

  while (++ i < len) { 
    
       /* xor the next byte from input (remember: x == x^0) */ 
       accu ^= (* buf ++ & 0x00ff);

       j = 8;
       while (j --) {
	 /* shift right through CARRY, 
	    xor the polynomial if the CARRY is set */
	 if (accu & 0x0001) { 
	   accu >>= 1; 
	   accu  ^= CRC16_POLYNOMIAL; 
           continue ; 
	 }
         accu >>= 1; 
       } 
  } 
  *(unsigned *)Accu = accu ;
} 
#endif /* CRC16_FRAME */


/* ---------------------------------------------------------------------------- *
 *                   interfaces                                                 *
 * ---------------------------------------------------------------------------- */

#ifdef CIPHER_PLAIN
const char *
plain_get_info (int algo, size_t *keylen,
		size_t *blocksize, size_t *contextsize,
		int	(**r_setkey)( void *c, byte *key, unsigned keylen ),
		void (**r_encrypt)( void *c, byte *outbuf, byte *inbuf ),
		void (**r_decrypt)( void *c, byte *outbuf, byte *inbuf )
		)
{
  switch (algo) {
  case 8:
    *keylen      =     128 ;
    *blocksize   =       8 ;
    *contextsize =       3 ;
    *r_setkey    = &set_key ;
    *r_encrypt   = &plain8 ;
    *r_decrypt   = &plain8 ;
    return "PLAIN8" ;

  case -8:
    *keylen      =      128 ;
    *blocksize   =        8 ;
    *contextsize =        3 ;
    *r_setkey    =  &set_key ;
    *r_encrypt   = &invert8 ;
    *r_decrypt   = &invert8 ;
    return "INVERT8" ;

  case 16:
    *keylen      =      128 ;
    *blocksize   =       16 ;
    *contextsize =        3 ;
    *r_setkey    =  &set_key ;
    *r_encrypt   = &plain16 ;
    *r_decrypt   = &plain16 ;
    return "PLAIN16" ;

  case -16:
    *keylen      =       128 ;
    *blocksize   =        16 ;
    *contextsize =         3 ;
    *r_setkey    =   &set_key ;
    *r_encrypt   = &invert16 ;
    *r_decrypt   = &invert16 ;
    return "INVERT16" ;
  }

  return NULL;
}
#endif /* CIPHER_PLAIN */

#ifdef CRC16_FRAME
const char *
crc16_get_info(int algo, size_t *contextsize,
	     byte **r_asnoid, int *r_asnlen, int *r_mdlen,
	     void (**r_init) (void *),
	     void (**r_write)(void *, byte *buf, size_t nbytes),
	     void (**r_final)(void *),
	     byte *(**r_read)(void *)
	     )
{
  *contextsize = sizeof (unsigned);
  *r_asnoid = 0;
  *r_asnlen = 0 ;
  *r_mdlen = sizeof (unsigned);
  *r_init  = crc16_init;
  *r_write = crc16_write;
  *r_final = crc16_final;
  *r_read  = crc16_read;
  return "CRC16";
}
#endif /* CRC16_FRAME */
